#!/bin/sh
#
# Usage: install-ciphire.sh [system-dir]
# Version: 1.2 (047)
# This script installs the Ciphire Mail binary distribution.
# Copyright 2003-2006 Ciphire Labs

CITARGET_ROOT_DEFAULT=/usr/local/ciphire
USERID=`id -u`
GROUPID=`id -g`
DATE=`date`
CN=`basename $0`
EXPERT=
XSUPPORT=1
CIDIR="./files"

cd `dirname $0`

if [ -e "files/setupdata/libciphire.sh" ]
then
  . ./files/setupdata/libciphire.sh
else
  echo "ERROR: libciphire.sh not found!"
  exit 1
fi

writelog "Starting installation [`uname -a`]"

set_gw

if set_popecho
then
  HAVEDISPLAY=1
else
  HAVEDISPLAY=0
fi

writelog "Setting up language configuration."

set_lang_config $HAVEDISPLAY
set_lang

writelog "Chosen language: $UT_LANG"

UT_BUTTON_YES="`getloc CI_BUTTON_YES`"
UT_BUTTON_NO="`getloc CI_BUTTON_NO`"
UT_BUTTON_CANCEL="`getloc CI_BUTTON_CANCEL`"
UT_BUTTON_ABORT="`getloc CI_BUTTON_ABORT`"
UT_BUTTON_CONTINUE="`getloc CI_BUTTON_CONTINUE`"

for DL in /usr/lib/libdl.so /lib/libdl.so /lib/libdl.so.2 /usr/lib/libdl.so.2 ""
do
  test -f $DL || continue
  LD_PRELOAD=$DL:files/ciphire-redir.so /bin/true 2>/dev/null || continue
  PRELOAD_VALUE=$DL:files/ciphire-redir.so
  break
done
if ! LD_PRELOAD=$PRELOAD_VALUE /bin/true 2>/dev/null
then
  writelog "Preload check failed. Value=$PRELOAD_VALUE"
  popecho error "`getloc UT_WRONG_GLIBC`" error
fi
unset LD_PRELOAD

if [ "$GW" ]
then
  I_START="`getloc UT_INST_START`"
  popecho cont "$I_START" qm
  EXPERT=1
else
  I_START="`getloc UT_INST_START_EXPERT`"
  popecho yesno "$I_START" qm
  if [ "$?" = "1" ]
  then
    EXPERT=1
  fi
fi


popecho license "`getloc UT_INST_ACCEPT_LICENSE`" qm


if [ "x$1" = "x" ]
then
 if [ ! "$EXPERT" ]
  then
    CUSTOM_SYS=
  else
    popecho input "`getloc UT_INST_WHERE_SYSDIR`" qm $CITARGET_ROOT_DEFAULT
    CUSTOM_SYS=$INPUT_VALUE
  fi
  if [ "x$CUSTOM_SYS" = "x" ] 
  then
    CITARGET=$CITARGET_ROOT_DEFAULT
  else
    CITARGET=$CUSTOM_SYS
  fi
else
  CITARGET=$1
fi

if [ "$EXPERT" ]
then
  popecho cont "`getloc UT_INST_SHOW_SYSDIR`" info
  if [ ! "$GW" ]
  then
    popecho yesno "`getloc UT_INST_WANT_X11`" qm
    if [ "$?" = "1" ]
    then
      XSUPPORT=0
    fi
  fi
fi

writelog "Going to install in $CITARGET [XSUPPORT=$XSUPPORT]"

if [ "$USERID" = "0" ]
then
  writelog "Invoking ciphire-systemsetup as root (direct: $CITARGET $XSUPPORT $USERID $GROUPID $HAVEDISPLAY)"
  ./files/ciphire-systemsetup $CITARGET $XSUPPORT $USERID $GROUPID $HAVEDISPLAY
else
  NEEDROOT=1
  if [ ! "$GW" ]
  then
    if [ -e "$CITARGET" ]
    then
      if [ -w "$CITARGET" -a "x$XSUPPORT" = "x0" ]
      then
        NEEDROOT=
      fi
    else
      if [ -w "`dirname $CITARGET`" -a "x$XSUPPORT" = "x0" ]
      then
        NEEDROOT=
      fi
    fi
  fi
  if [ "$NEEDROOT" ]
  then
    writelog "Need root for ciphire-systemsetup ($CITARGET $XSUPPORT $USERID $GROUPID $HAVEDISPLAY $NEEDROOT)"
    ./files/ciphire-systemsetup $CITARGET $XSUPPORT $USERID $GROUPID $HAVEDISPLAY $NEEDROOT
  else
    writelog "Invoking ciphire-systemsetup as user ($CITARGET $XSUPPORT $USERID $GROUPID $HAVEDISPLAY)"
    ./files/ciphire-systemsetup $CITARGET $XSUPPORT $USERID $GROUPID $HAVEDISPLAY
  fi
fi

if [ ! -e $CITARGET/ciphire-mail -o \( "$?" != "0" \) ]
then
  popecho error "`getloc UT_INST_FAILED`" error
fi

FINISH_LOC_HEADER="`getloc UT_GEN_FINISH_HEADER`"
FINISH_HEADER="${FINISH_LOC_HEADER}(Logfile: /tmp/install-ciphire.log)\n"
FINISH_X1="`getloc UT_INST_FINISH_X11_INSTALLED`"
FINISH_X0="`getloc UT_INST_FINISH_X11_NOT_INSTALLED`"
FINISH_P1="`getloc UT_INST_FINISH_USERS_SETUP`"
FINISH_P0="`getloc UT_INST_FINISH_USERS_NOT_SETUP`"
FINISH_FOOTER="`getloc UT_GEN_FINISH_FOOTER`"

if [ ! "$GW" ]
then
  if [ "$USERID" != "0" ]
  then
    if [ "$EXPERT" ]
    then
      writelog "Invoking ciphire-setup 1 $HAVEDISPLAY"
      $CITARGET/ciphire-setup 1 $HAVEDISPLAY
    else
      writelog "Invoking ciphire-setup 0 $HAVEDISPLAY"
      $CITARGET/ciphire-setup 0 $HAVEDISPLAY
    fi
  else
    popecho yesno "`getloc UT_INST_WANT_USER_SETUP`" qm
    if [ "$?" = "0" ]
    then
      popecho input "`getloc UT_INST_WHICH_USER_SETUP`" ""
      NEWUSERS=$INPUT_VALUE
      if [ "$NEWUSERS" ]
      then
        writelog "Processing user accounts: $NEWUSERS"
        PROCESSED_ACCOUNTS=1
        PROCESSED_OUTPUT="`getloc UT_INST_PROCESSED_ACCOUNTS`"
        for i in `echo "$NEWUSERS"`
        do
          UHOME=`su - $i -c "echo ~" 2>/dev/null`
          if [ ! "x$UHOME" = "x" ]
          then
            writelog "Invoking ciphire-setup 0 0 [user=$i]"
            su - $i -c "$CITARGET/ciphire-setup 0 0" >/dev/null
            PROCESSED_OUTPUT="${PROCESSED_OUTPUT}\n- ${i}: `getloc UT_INST_USER_CONFIG_PROCESSED`"
          else
            PROCESSED_OUTPUT="${PROCESSED_OUTPUT}\n- ${i}: `getloc UT_INST_NO_USER`"
          fi
        done
        popecho nop "${PROCESSED_OUTPUT}\n" info
        writelog "Finished processing user accounts."
      fi
    fi

    if [ "x$XSUPPORT" = "x1" ]
    then
      FINISH="${FINISH_HEADER}${FINISH_X1}"
    else
      FINISH="${FINISH_HEADER}${FINISH_X0}"
    fi
    if [ "x$PROCESSED_ACCOUNTS" = "x1" ]
    then
      FINISH="${FINISH}${FINISH_P1}"
    else
      FINISH="${FINISH}${FINISH_P0}"
    fi
    FINISH="${FINISH}${FINISH_FOOTER}"
    popecho nop "$FINISH" info
  fi
else
  FINISH="${FINISH_HEADER}${FINISH_FOOTER}"
  popecho nop "$FINISH" info
fi

writelog "Finished installation."
